﻿# -*- coding: utf-8 -*-
"""
Reads the template language XML and translates the template.
Loads the TemplateTranslationDefs[<language>].xml file located in the same directory as the script.

MakeTranslateList uses .NET classes.
"""

import sys
import clr
import cse
import re
clr.AddReference("System.Xml")
import System.Xml.XmlDocument
import System.IO.Path as path
import System.IO.File as file

# Language information dictionary.
langlist = None
sheetlist = None

# Regular expression objects.
rx = re.compile(r"(\.[tc]\.\w+)\.(.+)$", re.I | re.L)
valrx = re.compile(r"^value\[(.+)\]$", re.I | re.L)

# The class storing sheet information.
class SheetVersionInfo:
  "The class storing sheet information."
  def __init__(self, Name, Title, MjVer, MnVer):
    self.Name = Name
    self.Title = Title
    self.MjVer = MjVer
    self.MnVer = MnVer


# Method for reading language XML.
def MakeTranslateList(xmlPath):
  global langlist
  global sheetlist
  
  # Loads XML language code.
  outDic = {}	# Dictionary with csePath as the key and the value set to "translation key & translated sentence dictionary."
  sheetDic = {}	# Dictionary with the sheet name as the key and the value set to the sheet title or something.
  
  xml = System.Xml.XmlDocument()
  xml.Load(xmlPath)
  
  root = xml.SelectSingleNode("/cseTemplateLanguageDefinitions")
  langname = root.Attributes.GetNamedItem("language").Value
  
  for sheetnode in root.SelectNodes("sheet"):
    # Gets the attributes of the sheet node.
    try:
      sheetName = sheetnode.Attributes.GetNamedItem("name").Value
      sheetTitle = sheetnode.Attributes.GetNamedItem("title").Value
      sheetMjVer = sheetnode.Attributes.GetNamedItem("majorVer").Value
      sheetMnVer = sheetnode.Attributes.GetNamedItem("minorVer").Value
    except AttributeError, e:
      sys.stderr.write("'sheet' node doesn't have enough attributes.\n")
      continue
    
    # Creates a sheet information object.
    try:
      sheetMjVerNum = int(sheetMjVer)
      sheetMnVerNum = int(sheetMnVer)
    except ValueError, e:
      sys.stderr.write(str(e) + "\n")
      continue 
    sheetDic[sheetName] = SheetVersionInfo(sheetName, sheetTitle, sheetMjVerNum, sheetMnVerNum)
    
    for itemnode in sheetnode.SelectNodes("lngitem"):
      try:
        localePath = itemnode.Attributes.GetNamedItem("key").Value
      except AttributeError, e:
        sys.stderr.write("'lngitem' node has no attribute 'key'.\n")
        continue
      
      Value = itemnode.InnerText
      
      # Cuts out the CSE path portion.
      m = rx.match(localePath)
      Path = sheetName + m.group(1)
      ItemPath = m.group(2)
      
      # If the Path dictionary does not exist, creates one.
      if Path not in outDic:
        outDic[Path] = {}
      
      # Add
      outDic[Path][ItemPath] = Value
      
    
  langlist = outDic
  sheetlist = sheetDic


# Loads the language information file.
def LoadTranslationFile():
  global langlist
  
  xmlname = "TemplateTranslationDefs[%s].xml" % cse.appsetting.CurrentLngName
  
  # Gets the folder name.
  xmlpath = path.Combine(path.GetDirectoryName(__file__), xmlname)
  
  # Loads the file dictionary.
  if file.Exists(xmlpath):
    MakeTranslateList(xmlpath)



# Method for checking output sheets.
def TranslateTemplates(s, e):
  global langlist
  global sheetlist
  if langlist == None or sheetlist == None:
    return
  
  # Plugin execution event data.
  sheetTabItems = e.Value
  
  for sti in sheetTabItems:
    # Compares versions and does not translate if they differ.
    sheetValue = sti.Sheet
    if sheetValue.Name not in sheetlist:
      sys.stderr.write("'" + sheetValue.Name + "' is not defined.\n")
      continue
    svi = sheetlist[sheetValue.Name]
    if sheetValue.MajorVer != svi.MjVer or sheetValue.MinorVer != svi.MnVer:
      sys.stderr.write("'%s' ver. %d.%d is not defined.\n" % (sheetValue.Name, sheetValue.MajorVer, sheetValue.MinorVer))
      continue
    
    # Applies the title.
    sheetValue.Title = svi.Title
    
    # Loops on content defined on parent side and child side.
    for contset in cse.getcontssets(sheetValue, True):
      for cont in contset:
        # Gets the CSE path string.
        path = cse.deparsepath(cont)
        
        if path in langlist:
          # If the CSE path key exists in the language dictionary, replaces text using that translation key and each translated sentence.
          #
          for k, v in langlist[path].items():
            if k.lower() == "title":
              cont.Title = v
            elif k.lower() == "note":
              cont.Note = v
            elif k.lower() == "value":
              # A value with no index.
              if str(cont.type) == "number":
                # Numeric value
                try:
                  cont.Value = int(v)
                except:
                  pass
                
              elif str(cont.type) == "text" or str(cont.type) == "multitext":
                # Text
                cont.Value = v
                
            elif valrx.match(k) != None:
              # A value with an index.
              index = valrx.match(k).group(1)
              
              if str(cont.type) == "list":
                # List
                try:
                  index = int(index)
                except:
                  continue
                if 0 <= index < len(cont.Value):
                  cont.Value[index] = v
                
              elif str(cont.type) == "check" or str(cont.type) == "radio":
                # Check box or radio button.
                for chk in cont.Value:
                  if chk.Id == index:
                    chk.Text = v
                
    


# Registers event handler when loading of XML is run.
LoadTranslationFile()
cse.addhandler("TemplateLoaded", TranslateTemplates)

